@@ -1,5 +1,5 @@
 # 
-# Copyright (C) 2010 Platform Computing
+# Copyright (C) 2010-2011 Platform Computing
 # 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,8 @@ Created on Nov 10, 2010
 
 # disabling 'Too few public methods' pylint check R0903 (This is a model)
 # disabling 'Missing docstring' pylint check C0111 (Docs can be found in OCCI)
-# pylint: disable=R0903,C0111
+# disabling 'Too many instance attributes' pyling check Ro902 (All okay)
+# pylint: disable=R0903,C0111,R0902
 
 class Category(object):
 
@@ -35,6 +36,8 @@ class Category(object):
         self.title = ''
         self.attributes = []
 
+        self.cls_str = 'action'
+
     def __eq__(self, instance):
         if instance is None or not isinstance(instance, Category):
             return False
@@ -55,6 +58,7 @@ class Kind(Category):
 
         # Following are not defined by the OCCI spec
         self.location = ''
+        self.cls_str = 'kind'
 
 class Mixin(Category):
 
@@ -66,6 +70,7 @@ class Mixin(Category):
         # Following are not defined by the OCCI spec
         self.location = ''
         self.owner = ''
+        self.cls_str = 'mixin'
 
 class Entity(object):
 
@@ -145,7 +150,7 @@ class Action(object):
         self.attributes = {}
 
     def __repr__(self):
-        return self.kind.term
+        return self.kind.scheme + '#' + self.kind.term
 
     def __eq__(self, instance):
         if instance is None or not isinstance(instance, Action):
