@@ -1,5 +1,5 @@
 # 
-# Copyright (C) 2010 Platform Computing
+# Copyright (C) 2010-2011 Platform Computing
 # 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -149,6 +149,9 @@ def _get_category(cat_string):
     @param cat_string: The string with RFC compliant syntac of a Category.
     @type cat_string: str
     '''
+
+    # TODO: make use of the class attribute
+
     cat = Category()
     # find the term
     tmp = cat_string.split(';')
@@ -166,21 +169,21 @@ def _get_category(cat_string):
         end = tmp.find(";")
         if end > -1:
             tmp = tmp[:end]
-        scheme = tmp.rstrip("\"").lstrip("\"")
+        scheme = tmp.rstrip('"').lstrip('"')
         if scheme.find('http') is not - 1:
-            cat.scheme = scheme.strip()
+            cat.scheme = scheme.strip().rstrip('#')
         else:
             raise ParsingException('No valid scheme for given category'
                                    + ' could be found.')
 
-    # find the scheme
+    # find the location
     begin = cat_string.find('location=')
     if begin is not - 1:
         tmp = cat_string[begin + 9:]
         end = tmp.find(";")
         if end > -1:
             tmp = tmp[:end]
-        location = tmp.rstrip("\"").lstrip("\"")
+        location = tmp.rstrip('"').lstrip('"')
         tmp = location
         if tmp.find('/') is 0 and tmp.rfind('/') is len(tmp) - 1:
             res = Mixin()
@@ -239,7 +242,7 @@ def _get_attributes(attributes):
                 raise ParsingException('Could not determine the attributes...')
     return tmp
 
-def _create_categories(kind, extended = False):
+def _from_category(kind, extended = False):
     '''
     Create a category rendering for a kind or mixin. If extended it will try to
     put in as much information as possible.
@@ -247,30 +250,117 @@ def _create_categories(kind, extended = False):
     @param kind: A kind of a mixin.
     @type kind: Kind or Mixin
     @param extended: Indicating wether rendering should be minimal or not.
-    @param extended: boolean  
+    @type extended: boolean  
     '''
     tmp = ''
     tmp += kind.term
-    tmp += ';scheme=' + kind.scheme
+    tmp += ';scheme="' + kind.scheme + '#"'
+    tmp += ';class="' + kind.cls_str + '"'
     if extended:
         if hasattr(kind, 'title') and kind.title is not '':
-            tmp += ';title=' + kind.title
-        if hasattr(kind, 'location') and kind.location is not '':
-            tmp += ';location=' + kind.location
+            tmp += ';title="' + kind.title + '"'
         if hasattr(kind, 'related') and len(kind.related) > 0:
             rel_list = []
             for item in kind.related:
                 rel_list.append(repr(item))
             tmp += ';rel=' + ' '.join(rel_list)
+        if hasattr(kind, 'location') and kind.location is not '':
+            tmp += ';location=' + kind.location
         if hasattr(kind, 'attributes') and len(kind.attributes) > 0:
-            tmp += ';attributes=' + ' '.join(kind.attributes)
+            tmp += ';attributes="' + ' '.join(kind.attributes) + '"'
         if hasattr(kind, 'actions') and len(kind.actions) > 0:
             action_list = []
             for item in kind.actions:
                 action_list.append(repr(item))
-            tmp += ';actions=' + ' '.join(action_list)
+            tmp += ';actions="' + ' '.join(action_list) + '"'
     return tmp
 
+def _to_entity(defined_kind, kind, categories, attributes, allow_incomplete):
+    '''
+    Helper routine for creating a new entity.
+    
+    @param defined_kind: A previsously defined kind.
+    @type defined_kind: Kind
+    @param kind: A kind
+    @type kind: Kind
+    @param categories: A list of all categories.
+    @type categories: list
+    @param attributes: A list of all attributes.
+    @type attributes: list
+    @param allow_incomplete: Indicates wether rendering should be complete.
+    @type allow_incomplete: boolean
+    '''
+    if allow_incomplete:
+        kind = defined_kind
+    elif kind is None:
+        raise ParsingException('Could not find a Kind for this request.')
+
+    if Resource.category in kind.related:
+        entity = Resource()
+        if 'summary' in attributes.keys():
+            entity.summary = attributes['summary']
+            attributes.pop('summary')
+    elif Link.category in kind.related:
+        entity = Link()
+        if 'source' in attributes.keys():
+            source = attributes['source']
+            if source.find(registry.HOST) == 0:
+                entity.source = source[len(registry.HOST):]
+            else:
+                entity.source = attributes['source']
+            attributes.pop('source')
+        if 'target' in attributes.keys():
+            target = attributes['target']
+            if target.find(registry.HOST) == 0:
+                entity.target = target[len(registry.HOST):]
+            else:
+                entity.target = target
+            attributes.pop('target')
+    entity.kind = kind
+    entity.mixins = categories
+    entity.attributes = attributes
+    return entity
+
+def _from_entity(entity):
+    '''
+    Helper routine for rendering an entity.
+    
+    @param entity: The entity to create a rendering for.
+    @type entity: Entity
+    '''
+    result = HTTPData()
+
+    # add kind
+    result.categories.append(_from_category(entity.kind))
+    # mixins...
+    for item in entity.mixins:
+        result.categories.append(_from_category(item))
+
+    if isinstance(entity, Resource):
+        # check if summary is available...
+        if entity.summary is not '':
+            result.attributes.append('summary=' + entity.summary)
+
+        if len(entity.actions) > 0:
+            for action in entity.actions:
+                result.links.append('<' + registry.HOST + entity.identifier
+                                    + '?action=' + action.kind.term + '>')
+
+        if len(entity.links) > 0:
+            for item in entity.links:
+                result.links.append('<' + registry.HOST + item.target
+                                    + '>;self=' + registry.HOST
+                                    + item.identifier + ';')
+
+    elif isinstance(entity, Link):
+        # source and target must be there!
+        result.attributes.append('source=' + registry.HOST + entity.source)
+        result.attributes.append('target=' + registry.HOST + entity.target)
+    for item in entity.attributes.keys():
+        result.attributes.append(item + '=' + entity.attributes[item])
+
+    return result
+
 #===============================================================================
 # Renderings for different content types can be found below this comment
 #===============================================================================
@@ -308,7 +398,7 @@ class TextPlainRendering(Rendering):
         headers = {}
         body = ''
         for item in categories:
-            body += 'Category: ' + _create_categories(item, extended = True)
+            body += 'Category: ' + _from_category(item, extended = True)
             body += '\n'
         headers['Content-Type'] = self.content_type
         return headers, body
@@ -317,45 +407,25 @@ class TextPlainRendering(Rendering):
         headers = {}
         body = ''
         for item in entities:
-            body += 'X-OCCI-Location: ' + item.identifier + '\n'
+            body += 'X-OCCI-Location: ' + registry.HOST + item.identifier + '\n'
         headers['Content-Type'] = self.content_type
         return headers, body
 
     def from_entity(self, entity):
-        data = ''
+        data = _from_entity(entity)
+        body = ''
 
-        # add kind
-        data += 'Category: ' + _create_categories(entity.kind) + '\n'
-        # mixins...
-        for item in entity.mixins:
-            data += 'Category: ' + _create_categories(item) + '\n'
-
-        if isinstance(entity, Resource):
-            # check if summary is available...
-            if entity.summary is not '':
-                data += 'X-OCCI-Attribute: summary=' + entity.summary + '\n'
-
-            if len(entity.actions) > 0:
-                for action in entity.actions:
-                    data += 'Link: <' + entity.identifier + '?action='
-                    data += action.kind.term + '>\n'
-
-            if len(entity.links) > 0:
-                for item in entity.links:
-                    data += 'Link: <' + item.target + '>;self='
-                    data += item.identifier + ';\n'
-
-        elif isinstance(entity, Link):
-            # source and target must be there!
-            data += 'X-OCCI-Attribute: source=' + entity.source + '\n'
-            data += 'X-OCCI-Attribute: target=' + entity.target + '\n'
-        for item in entity.attributes.keys():
-            data += 'X-OCCI-Attribute: ' + item + '='
-            data += entity.attributes[item] + '\n'
+        for item in data.categories:
+            body += 'Category: ' + item + '\n'
+        for item in data.attributes:
+            body += 'X-OCCI-Attribute: ' + item + '\n'
+        for item in data.links:
+            body += 'Link: ' + item + '\n'
 
         headers = {}
         headers['Content-Type'] = self.content_type
-        return headers, data
+        del(data)
+        return headers, body
 
     def get_entities(self, headers, body):
         ids = []
@@ -363,7 +433,10 @@ class TextPlainRendering(Rendering):
             if entry.find('X-OCCI-Location:') > -1:
                 tmp = entry[entry.find('X-OCCI-Location:') + 16:]
                 for item in tmp.split(','):
-                    ids.append(item.strip())
+                    if item.strip().find(registry.HOST) == 0:
+                        ids.append(item.strip()[len(registry.HOST):])
+                    else:
+                        ids.append(item.strip())
         if len(ids) > 0:
             return ids
         else:
@@ -422,28 +495,8 @@ class TextPlainRendering(Rendering):
         kind, categories = _get_categories(data.categories)
         attributes = _get_attributes(data.attributes)
 
-        if allow_incomplete:
-            kind = defined_kind
-        elif kind is None:
-            raise ParsingException('Could not find a Kind for this'
-                                   + ' request.')
-
-        if Resource.category in kind.related:
-            entity = Resource()
-            if 'summary' in attributes.keys():
-                entity.summary = attributes['summary']
-                attributes.pop('summary')
-        elif Link.category in kind.related:
-            entity = Link()
-            if 'source' in attributes.keys():
-                entity.source = attributes['source']
-                attributes.pop('source')
-            if 'target' in attributes.keys():
-                entity.target = attributes['target']
-                attributes.pop('target')
-        entity.kind = kind
-        entity.mixins = categories
-        entity.attributes = attributes
+        entity = _to_entity(defined_kind, kind, categories, attributes,
+                            allow_incomplete)
 
         del(data)
         return entity
@@ -481,7 +534,7 @@ class TextHeaderRendering(Rendering):
         headers = {}
         tmp = []
         for item in categories:
-            tmp.append(_create_categories(item, extended = True))
+            tmp.append(_from_category(item, extended = True))
         if len(tmp) > 0:
             headers['Category'] = ','.join(tmp)
         headers['Content-Type'] = self.content_type
@@ -491,7 +544,7 @@ class TextHeaderRendering(Rendering):
         headers = {}
         tmp = []
         for item in entities:
-            tmp.append(item.identifier)
+            tmp.append(registry.HOST + item.identifier)
         if len(tmp) > 0:
             headers['X-OCCI-Location'] = ','.join(tmp)
         headers['Content-Type'] = self.content_type
@@ -499,57 +552,27 @@ class TextHeaderRendering(Rendering):
 
     def from_entity(self, entity):
         headers = {}
+        data = _from_entity(entity)
 
-        # add kind
-        tmp = []
-
-        tmp.append(_create_categories(entity.kind))
-        # mixins...
-        for item in entity.mixins:
-            tmp.append(_create_categories(item))
-        if len(tmp) > 0:
-            headers['Category'] = ','.join(tmp)
-
-        attr_tmp = []
-        link_tmp = []
-
-        if isinstance(entity, Resource):
-            # check if summary is available...
-            if entity.summary is not '':
-                attr_tmp.append('summary=' + entity.summary)
-
-            if len(entity.actions) > 0:
-                for action in entity.actions:
-                    link_tmp.append('<' + entity.identifier + '?action='
-                                    + action.kind.term + '>')
-
-            if len(entity.links) > 0:
-                for item in entity.links:
-                    link_tmp.append('<' + item.target + '>;self='
-                                    + item.identifier + ';')
-
-        elif isinstance(entity, Link):
-            # source and target must be there!
-            attr_tmp.append('source=' + entity.source)
-            attr_tmp.append('target=' + entity.target)
-
-        # rest of the attributes.
-        for item in entity.attributes.keys():
-            attr_tmp.append(item + '=' + entity.attributes[item])
-
-        if len(attr_tmp) > 0:
-            headers['X-OCCI-Attribute'] = ','.join(attr_tmp)
-        if len(link_tmp) > 0:
-            headers['Link'] = ','.join(link_tmp)
+        if len(data.categories) > 0:
+            headers['Category'] = ','.join(data.categories)
+        if len(data.attributes) > 0:
+            headers['X-OCCI-Attribute'] = ','.join(data.attributes)
+        if len(data.links) > 0:
+            headers['Link'] = ','.join(data.links)
 
         headers['Content-Type'] = self.content_type
+        del(data)
         return headers, 'OK'
 
     def get_entities(self, headers, body):
         ids = []
         if 'X-OCCI-Location' in headers.keys():
             for item in headers['X-OCCI-Location'].split(','):
-                ids.append(item.strip())
+                if item.strip().find(registry.HOST) == 0:
+                    ids.append(item.strip()[len(registry.HOST):])
+                else:
+                    ids.append(item.strip())
         if len(ids) > 0:
             return ids
         else:
@@ -606,33 +629,58 @@ class TextHeaderRendering(Rendering):
         kind, categories = _get_categories(data.categories)
         attributes = _get_attributes(data.attributes)
 
-        # create a resource or link
-        if allow_incomplete:
-            kind = defined_kind
-        elif kind is None:
-            raise ParsingException('Could not find a Kind for this'
-                                   + ' request.')
-
-        if Resource.category in kind.related:
-            entity = Resource()
-            if 'summary' in attributes.keys():
-                entity.summary = attributes['summary']
-                attributes.pop('summary')
-        elif Link.category in kind.related:
-            entity = Link()
-            if 'source' in attributes.keys():
-                entity.source = attributes['source']
-                attributes.pop('source')
-            if 'target' in attributes.keys():
-                entity.target = attributes['target']
-                attributes.pop('target')
-        entity.kind = kind
-        entity.mixins = categories
-        entity.attributes = attributes
+        entity = _to_entity(defined_kind, kind, categories, attributes,
+                            allow_incomplete)
 
         del(data)
         return entity
 
+class URIListRendering(Rendering):
+    '''
+    This rendering can handle text/uri-list requests. Cannot be used for
+    creation of entities or similar.
+    '''
+
+    content_type = 'text/uri-list'
+    err_msg = 'text/uri-list rendering cannot be used for this operation.'
+
+    def from_categories(self, categories):
+        body = ''
+        for item in categories:
+            body += str(item) + '\n'
+
+        headers = {}
+        headers['Content-Type'] = self.content_type
+        return headers, body
+
+    def from_entities(self, entities):
+        body = ''
+        for item in entities:
+            body += registry.HOST + item.identifier + '\n'
+
+        headers = {}
+        headers['Content-Type'] = self.content_type
+        return headers, body
+
+    def from_entity(self, entity):
+        raise NotImplementedError(self.err_msg)
+
+    def get_entities(self, headers, data):
+        raise NotImplementedError(self.err_msg)
+
+    def login_information(self):
+        raise NotImplementedError(self.err_msg)
+
+    def to_action(self, headers, data):
+        raise NotImplementedError(self.err_msg)
+
+    def to_categories(self, headers, data):
+        raise NotImplementedError(self.err_msg)
+
+    def to_entity(self, headers, body, allow_incomplete = False,
+                  defined_kind = None):
+        raise NotImplementedError(self.err_msg)
+
 class TextHTMLRendering(Rendering):
     '''
     This rendering will use a generic HTML rendering to represent the resources.
@@ -640,7 +688,8 @@ class TextHTMLRendering(Rendering):
 
     # disabling 'Unused variable' pylint check (categories are not used all 
     #                                          the time)
-    # pylint: disable=W0612
+    # disabling 'Method could be a function' pylint check (I want it here)
+    # pylint: disable=W0612,R0201
 
     # FIXME: check if it wouldn't be better to do term;scheme=scheme instead 
     # of scheme#term
@@ -658,6 +707,43 @@ class TextHTMLRendering(Rendering):
         css_string += 'border: 1px solid #73c167;}'
         css_string += 'table {margin: 1em; border: 1px solid #444;}'
 
+    def _get_data(self, body):
+        '''
+        Simple method to split out the information from the HTTP HTML body.
+        
+        @param headers: The HTML body.
+        @type headers: str
+        '''
+        data = HTTPData()
+        if body is not None:
+            tmp = urllib.unquote(body).split('&')
+        else:
+            raise ParsingException('Body should not be empty.')
+
+        # FIXME!!! handle incomplete request properly!
+        try:
+            kin = tmp[0][tmp[0].find('Category=') + 9:].split('#')
+            kin = kin[1] + ';scheme=' + kin[0]
+        except IndexError:
+            raise ParsingException('Cannot find properely formatted data.')
+
+        attr = None
+        try:
+            test = tmp[1][tmp[1].find('X-OCCI-Attribute=') + 17:]
+            if test is '':
+                attr = []
+            else:
+                attr = test.split('+')
+        except IndexError:
+            pass
+
+        if kin is not None:
+            data.categories.append(kin)
+        if attr is not None:
+            data.attributes = attr
+
+        return data
+
     def from_categories(self, categories):
         html = self._create_html_head()
 
@@ -667,47 +753,12 @@ class TextHTMLRendering(Rendering):
 
         if len(categories) > 0:
             for cat in categories:
-                tmp = '<table>'
-                tmp += '<tr><th>Term'
-                tmp += '</th><td>'
-                tmp += cat.term + '</td></tr>'
-                tmp += '<tr><th>Scheme'
-                tmp += '</th><td>'
-                tmp += cat.scheme + '</td></tr>'
-
-                if hasattr(cat, 'title') and cat.title is not '':
-                    tmp += '<tr><th>Title'
-                    tmp += '</th><td>'
-                    tmp += cat.title + '</td></tr>'
-                if hasattr(cat, 'actions') and len(cat.actions) > 0:
-                    tmp += '<tr><th>Actions'
-                    tmp += '</th><td><ul>'
-                    for item in cat.actions:
-                        tmp += '<li>' + item.kind.term + '</li>'
-                    tmp += '</ul></td></tr>'
-                if hasattr(cat, 'attributes')and len(cat.attributes) > 0:
-                    tmp += '<tr><th>Attributes'
-                    tmp += '</th><td><ul>'
-                    for item in cat.attributes:
-                        tmp += '<li>' + item + '</li>'
-                    tmp += '</ul></td></tr>'
-                if hasattr(cat, 'location') and cat.location is not '':
-                    tmp += '<tr><th>Location'
-                    tmp += '</th><td><a href="'
-                    tmp += cat.location + '">'
-                    tmp += cat.location + '</a></td></tr>'
-                if hasattr(cat, 'related')and len(cat.related) > 0:
-                    tmp += '<tr><th>related'
-                    tmp += '</th><td><ul>'
-                    for item in cat.related:
-                        tmp += '<li>' + repr(item) + '</li>'
-                    tmp += '</ul></td></tr>'
-
-                tmp += '</table>'
+                tmp = self._create_category_table(cat)
                 if sorted_cats.has_key(cat.scheme):
                     sorted_cats[cat.scheme] = sorted_cats[cat.scheme] + tmp
                 else:
                     sorted_cats[cat.scheme] = tmp
+
             for scheme in sorted_cats:
                 html += '<h2>' + scheme + '</h2>'
                 html += sorted_cats[scheme]
@@ -831,20 +882,7 @@ class TextHTMLRendering(Rendering):
         return headers, html
 
     def to_action(self, headers, body):
-        if body is not None:
-            tmp = urllib.unquote(body).split('&')
-        else:
-            raise ParsingException('No valid data could be found.')
-
-        try:
-            kin = tmp[0][tmp[0].find("Category=") + 9:].split('#')
-            kin = kin[1] + ';scheme=' + kin[0]
-        except IndexError:
-            raise ParsingException('Could not find properly designed data.')
-
-        data = HTTPData()
-        if kin:
-            data.categories.append(kin)
+        data = self._get_data(body)
 
         # all data...
         kind, categories = _get_categories(data.categories)
@@ -873,28 +911,7 @@ class TextHTMLRendering(Rendering):
             raise ParsingException('When allowing an incomplete requests'
                                    + ' the kind must be predefined!')
 
-        if body is not None:
-            tmp = urllib.unquote(body).split('&')
-        else:
-            raise ParsingException('No valid data could be found.')
-
-        # FIXME!!! handle incomplete request properly!
-        try:
-            kin = tmp[0][tmp[0].find('Category=') + 9:].split('#')
-            kin = kin[1] + ';scheme=' + kin[0]
-            test = tmp[1][tmp[1].find('X-OCCI-Attribute=') + 17:]
-            if test is '':
-                attr = []
-            else:
-                attr = test.split('+')
-        except IndexError:
-            raise ParsingException('Cannot find properely formatted data.')
-
-        data = HTTPData()
-        if kin:
-            data.categories.append(kin)
-        if attr:
-            data.attributes = attr
+        data = self._get_data(body)
 
         # all data...
         kind, categories = _get_categories(data.categories)
@@ -911,10 +928,18 @@ class TextHTMLRendering(Rendering):
         elif Link.category in kind.related:
             entity = Link()
             if 'source' in attributes.keys():
-                entity.source = attributes['source']
+                source = attributes['source']
+                if source.find(registry.HOST) == 0:
+                    entity.source = source[len(registry.HOST):]
+                else:
+                    entity.source = attributes['source']
                 attributes.pop('source')
             if 'target' in attributes.keys():
-                entity.target = attributes['target']
+                target = attributes['target']
+                if target.find(registry.HOST) == 0:
+                    entity.target = target[len(registry.HOST):]
+                else:
+                    entity.target = target
                 attributes.pop('target')
         entity.kind = kind
         entity.attributes = attributes
@@ -929,6 +954,39 @@ class TextHTMLRendering(Rendering):
     # disabling 'Method could be a function' pylint check (these belong here!)
     # pylint: disable=R0201
 
+    def _create_category_table(self, category):
+        '''
+        Create a string containing a table with the category's information.
+        '''
+        tmp = '<table>'
+        tmp += '<tr><th>Term</th><td>' + category.term + '</td></tr>'
+        tmp += '<tr><th>Scheme</th><td>' + category.scheme + '</td></tr>'
+        tmp += '<tr><th>Class</th><td>' + category.cls_str + '</td></tr>'
+
+        if hasattr(category, 'title') and category.title is not '':
+            tmp += '<tr><th>Title/th><td>' + category.title + '</td></tr>'
+        if hasattr(category, 'actions') and len(category.actions) > 0:
+            tmp += '<tr><th>Actions</th><td><ul>'
+            for item in category.actions:
+                tmp += '<li>' + item.kind.term + '</li>'
+            tmp += '</ul></td></tr>'
+        if hasattr(category, 'attributes') and len(category.attributes) > 0:
+            tmp += '<tr><th>Attributes</th><td><ul>'
+            for item in category.attributes:
+                tmp += '<li>' + item + '</li>'
+            tmp += '</ul></td></tr>'
+        if hasattr(category, 'location') and category.location is not '':
+            tmp += '<tr><th>Location</th><td><a href="'
+            tmp += category.location + '">'
+            tmp += category.location + '</a></td></tr>'
+        if hasattr(category, 'related')and len(category.related) > 0:
+            tmp += '<tr><th>related</th><td><ul>'
+            for item in category.related:
+                tmp += '<li>' + repr(item) + '</li>'
+            tmp += '</ul></td></tr>'
+        tmp += '</table>'
+        return tmp
+
     def _create_form(self):
         '''
         Will create an HTML representation of a fieldset for creating resource
