@@ -1,5 +1,5 @@
 # 
-# Copyright (C) 2010 Platform Computing
+# Copyright (C) 2010-2011 Platform Computing
 # 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -43,10 +43,17 @@ class BaseHandler(tornado.web.RequestHandler):
 
     # disabling 'Unused argument' pylint check (overwritten methods)
     # disabling 'Too many public methods' pylint check (tornado's fault)
-    # pylint: disable=R0904,W0613
+    # disabling 'method could be a function' pylint check (for sanity reasons)
+    # pylint: disable=R0904,W0613,R0201
 
     version = 'pyocci OCCI/1.1'
 
+    def __init__(self, application, request, transforms = None):
+        if registry.HOST == '':
+            registry.HOST = request.protocol + '://' + request.host
+        super(BaseHandler, self).__init__(application, request,
+                                          transforms = None)
+
     def extract_http_data(self):
         '''
         Extracts all necessary information from the HTTP envelop. Minimize the
@@ -109,6 +116,43 @@ class BaseHandler(tornado.web.RequestHandler):
         else:
             return 'default'
 
+    def filter_childs(self, key, resources, categories):
+        '''
+        Retrieve childs in a hierachy and use categories to filter the result.
+        
+        @param key: A key.
+        @type key: str
+        @param resources: A list of resources.
+        @type resources: list
+        @param categories: A list of categories which is used to filter.
+        @type categories: list
+        '''
+        tmp = []
+        for name in resources:
+            if name.identifier.find(key) > -1 and key.endswith('/'):
+                if categories is None:
+                    tmp.append(name)
+                elif len(categories) > 0:
+                    for category in categories:
+                        if category == name.kind:
+                            tmp.append(name)
+                        elif category in name.kind.related:
+                            tmp.append(name)
+                        elif category in name.mixins:
+                            tmp.append(name)
+        return tmp
+
+    def get_my_resources(self):
+        '''
+        Returns a list of all resources belonging to the current user.
+        '''
+        my_resources = []
+        for tmp in RESOURCES.keys():
+            item = RESOURCES[tmp]
+            if item.owner == self.get_current_user():
+                my_resources.append(item)
+        return my_resources
+
 class ResourceHandler(BaseHandler):
     '''
     Handles basic HTTP operations. To achieve this it will make use of WSGI and
@@ -126,7 +170,6 @@ class ResourceHandler(BaseHandler):
         headers, body = self.extract_http_data()
         parser = self.get_pyocci_parser('Content-Type')
 
-
         if self.request.uri.find('?action=') > -1:
             key = self.request.uri.split('?')[0]
             try:
@@ -149,7 +192,7 @@ class ResourceHandler(BaseHandler):
             entity.owner = self.get_current_user()
             key = self._create_key(entity)
             RESOURCES[key] = entity
-        self._send_response({'Location': key}, 'OK')
+        self._send_response({'Location': registry.HOST + key}, 'OK')
 
     @tornado.web.authenticated
     def put(self, key):
@@ -189,6 +232,7 @@ class ResourceHandler(BaseHandler):
     def get(self, key):
         # find a parser for the data format the client provided...
         parser = self.get_pyocci_parser('Accept')
+        headers, body = self.extract_http_data()
 
         if key in RESOURCES.keys():
             entity = RESOURCES[key]
@@ -205,11 +249,16 @@ class ResourceHandler(BaseHandler):
             self._send_response(heads, data)
         elif key == '/':
             # render a list of resources...
-            resources = []
-            for item in RESOURCES.values():
-                if item.owner == self.get_current_user():
-                    resources.append(item)
+            categories = None
+            try:
+                data_parser = self.get_pyocci_parser('Content-Type')
+                categories = data_parser.to_categories(headers, body)
+            except (KeyError, ParsingException):
+                pass
 
+            my_resources = self.get_my_resources()
+
+            resources = self.filter_childs(key, my_resources, categories)
             heads, data = parser.from_entities(resources)
             self._send_response(heads, data)
         else:
@@ -273,17 +322,6 @@ class ListHandler(BaseHandler):
                 locations[cat.location] = cat
         return locations
 
-    def get_my_resources(self):
-        '''
-        Returns a list of all resources belonging to the current user.
-        '''
-        my_resources = []
-        for tmp in RESOURCES.keys():
-            item = RESOURCES[tmp]
-            if item.owner == self.get_current_user():
-                my_resources.append(item)
-        return my_resources
-
     def get_all_resources_of_category(self, category, resources):
         '''
         Return all resources belonging to one category.
@@ -301,32 +339,6 @@ class ListHandler(BaseHandler):
                 tmp.append(res)
         return tmp
 
-    def filter_childs(self, key, resources, categories):
-        '''
-        Retrieve childs in a hierachy and use categories to filter the result.
-        
-        @param key: A key.
-        @type key: str
-        @param resources: A list of resources.
-        @type resources: list
-        @param categories: A list of categories which is used to filter.
-        @type categories: list
-        '''
-        tmp = []
-        for name in resources:
-            if name.identifier.find(key) > -1 and key.endswith('/'):
-                if categories is None:
-                    tmp.append(name)
-                elif len(categories) > 0:
-                    for category in categories:
-                        if category == name.kind:
-                            tmp.append(name)
-                        elif category in name.kind.related:
-                            tmp.append(name)
-                        elif category in name.mixins:
-                            tmp.append(name)
-        return tmp
-
     @tornado.web.authenticated
     def get(self, key):
         key = '/' + key + '/'
@@ -414,7 +426,12 @@ class QueryHandler(BaseHandler):
 
     @tornado.web.authenticated
     def get(self):
-        parser = self.get_pyocci_parser('Accept')
+        headers, body = self.extract_http_data()
+        parser = self.get_pyocci_parser('Content-Type')
+        return_parser = self.get_pyocci_parser('Accept')
+
+        result = []
+
         cats = []
         for item in registry.BACKENDS.keys():
             if isinstance(item, Mixin):
@@ -423,7 +440,20 @@ class QueryHandler(BaseHandler):
             else:
                 cats.append(item)
 
-        heads, data = parser.from_categories(cats)
+        filter_cats = []
+        try:
+            filter_cats = parser.to_categories(headers, body)
+        except ParsingException:
+            pass
+
+        if len(filter_cats) == 0:
+            result = cats
+        else:
+            for item in cats:
+                if item in filter_cats:
+                    result.append(item)
+
+        heads, data = return_parser.from_categories(result)
         self._send_response(heads, data)
 
     @tornado.web.authenticated
